/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "directoryviewwidget.h"
#include "logger.h"

#include <QDir>
#include <QFileDialog>

DirectoryViewWidget::DirectoryViewWidget(QWidget* parent)
	: QWidget(parent)
{
	setupUi(this);

	path = QDir::currentPath();

	QStringList filters;
	filters << "*.flam3" << "*.flam" << "*.flame";
	model = new QDirModel(filters, QDir::Files | QDir::AllDirs,
						  QDir::Name | QDir::DirsFirst);
	icon_provider = new FlamFileIconProvider;


	m_dirListView->setIconSize(QSize(50,50));
	m_dirListView->setSpacing(2);
	m_dirListView->setModel(model);
	m_dirListView->setRootIndex(model->index(path));
	m_dirLineEdit->setText(path);

	connect(m_openButton, SIGNAL(clicked(bool)),
			this, SLOT(openDirectoryAction(bool)));
	connect(m_dirListView, SIGNAL(pressed(const QModelIndex&)),
			this, SLOT(selectFileAction(const QModelIndex&)));
}


void DirectoryViewWidget::openDirectoryAction(bool t)
{
	QString openDir = QFileDialog::getExistingDirectory(this, "Select a directory",
				QFileInfo(QDir::currentPath()).absolutePath(),
				QFileDialog::ShowDirsOnly);

	if (!openDir.isEmpty())
	{
		m_dirLineEdit->setText(openDir);
		m_dirListView->setRootIndex(model->index(openDir));
		path = openDir;
	}
}

void DirectoryViewWidget::selectFileAction(const QModelIndex& idx)
{
	if (model->isDir(idx))
	{
		logInfo("DirectoryViewWidget::selectFileAction : dir selected");
		path = model->fileInfo(idx).absoluteFilePath();
		m_dirLineEdit->setText(path);
		m_dirListView->setRootIndex(idx);
	}
	else
	{
		logInfo("DirectoryViewWidget::selectFileAction : file selected");
		emit flam3FileSelected(model->fileInfo(idx).absoluteFilePath());
	}
}

DirectoryViewWidget::~DirectoryViewWidget()
{
	delete icon_provider;
	delete model;
}

void DirectoryViewWidget::showEvent(QShowEvent* e)
{
	if (model->iconProvider() != icon_provider)
		model->setIconProvider(icon_provider);
	model->refresh();
}


void DirectoryViewWidget::setCurrentPath(QString& p)
{
	if (p.isEmpty()) return;
	QModelIndex i = model->index(p);
	if (i.isValid() && model->isDir(i))
	{
		path = p;
		logInfo("DirectoryViewWidget::setCurrentPath : dir selected");
		m_dirLineEdit->setText(path);
		m_dirListView->setRootIndex(i);
	}
}


QString DirectoryViewWidget::currentPath()
{
    return path;
}
