/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "doublevalueeditor.h"


DoubleValueEditor::DoubleValueEditor(QWidget* parent)
	: QDoubleSpinBox(parent), wheelEventSignal(true)
{
}

void DoubleValueEditor::updateValue(double v)
{
	QDoubleSpinBox::setValue(v);
}


void DoubleValueEditor::keyPressEvent(QKeyEvent* e)
{
	switch (e->key())
	{
		case Qt::Key_BracketLeft:
			setSingleStep(singleStep() * 10.0);
			break;

		case Qt::Key_BracketRight:
			setSingleStep(singleStep() * 0.10);
			break;

		case Qt::Key_Space:
		case Qt::Key_Return:
			emit valueUpdated();
			break;

		case Qt::Key_Backslash:
			WheelValueEditor::WHEEL_EVENTS_ENABLED
					= ! WheelValueEditor::WHEEL_EVENTS_ENABLED;
			break;

		default:
			QDoubleSpinBox::keyPressEvent(e);
	}
}

void DoubleValueEditor::wheelEvent(QWheelEvent* e)
{
	if (!QDoubleSpinBox::isActiveWindow())
		QDoubleSpinBox::activateWindow();
	QDoubleSpinBox::setFocus(Qt::MouseFocusReason);

	if (e->modifiers() & Qt::ShiftModifier)
	{
		double step = singleStep();
		setSingleStep(step / 10.0);
		QDoubleSpinBox::wheelEvent(e);
		setSingleStep(step);
	}
	else
		QDoubleSpinBox::wheelEvent(e);

	if (wheelEventSignal && WheelValueEditor::WHEEL_EVENTS_ENABLED)
		emit valueUpdated();
}

void DoubleValueEditor::mousePressEvent(QMouseEvent* e)
{
	if (e->button() == Qt::MidButton)
		emit valueUpdated();
	else
		QDoubleSpinBox::mousePressEvent(e);
}


bool DoubleValueEditor::wheelEventUpdate() const
{
	return wheelEventSignal;
}

void DoubleValueEditor::setWheelEventUpdate(bool value)
{
	wheelEventSignal = value;
}


