/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QColorDialog>

#include "edittrianglewidget.h"

#define genome_ptr (genome->selectedGenome())

EditTriangleWidget::EditTriangleWidget(GenomeVector* g, QWidget* parent)
	: QWidget(parent), genome(g)
{
	setupUi(this);

	connect(m_rotateSlider, SIGNAL(valueChanged(int)),
            this, SLOT(triangleRotatedAction()));
	connect(m_scaleSlider, SIGNAL(valueChanged(int)),
            this, SLOT(triangleScaledAction()));
	connect(m_hFlipButton, SIGNAL(pressed()),
			this, SLOT(triangleHFlippedAction()));
	connect(m_vFlipButton, SIGNAL(pressed()),
			this, SLOT(triangleVFlippedAction()));
	connect(m_sceneScaleSlider, SIGNAL(valueChanged(int)),
			this, SLOT(sceneScaledAction()));
	connect(m_gridCheckBox, SIGNAL(toggled(bool)),
			this, SLOT(toggleGridAction(bool)));
	connect(m_gridColorButton, SIGNAL(pressed()),
			this, SLOT(gridColorSelectAction()));
	connect(m_bgColorButton, SIGNAL(pressed()),
			this, SLOT(bgColorSelectAction()));
	connect(m_moveLeftButton, SIGNAL(pressed()),
			this, SLOT(moveTriangleLeftAction()));
	connect(m_moveRightButton, SIGNAL(pressed()),
			this, SLOT(moveTriangleRightAction()));
	connect(m_moveUpButton, SIGNAL(pressed()),
			this, SLOT(moveTriangleUpAction()));
	connect(m_moveDownButton, SIGNAL(pressed()),
			this, SLOT(moveTriangleDownAction()));
	connect(m_rotateLineEdit, SIGNAL(returnPressed()),
			this, SLOT(rotateTextEditedSlot()));
	connect(m_scaleLineEdit, SIGNAL(returnPressed()),
			this, SLOT(scaleTextEditedSlot()));
	connect(m_markButton, SIGNAL(toggled(bool)),
			this, SLOT(toggleMarkAction(bool)));

	connect(m_finalCheckBox, SIGNAL(stateChanged(int)),
			this, SLOT(finalStateChangedSlot(int)));
}

void EditTriangleWidget::triangleScaledAction()
{
	int value = m_scaleSlider->dx();
	double scale;
	if (value > 0)
	{
		scale = 1.05;
		scale_dx *= 1.05;
	}
	else if (value < 0)
	{
		scale = 0.95;
		scale_dx *= 0.95;
	}
	else
		return;

	QPointF pos(0.0, 0.0);
	if (m_triangleButton->isChecked())
		// get the triangle's center position from the scene
		pos = selectedTriangle->mapFromScene(
					selectedTriangle->sceneBoundingRect().center());
	else if (m_markButton->isChecked())
		// center on the marker
		pos = selectedTriangle->mapFromScene(
				selectedTriangle->editor()->mark()->center());

	m_scaleLineEdit->setText(QString::number(scale_dx, 'f', 3));
	selectedTriangle->scale(scale, scale, pos);
}


void EditTriangleWidget::triangleRotatedAction()
{
	int value = m_rotateSlider->dx();
	if (value == 0)
		return;

	double deg = -1.0 * value / 16.0;
	rotate_dx += deg;
	while (rotate_dx > 360.0)
		rotate_dx -= 360.0;
	while (rotate_dx < -360.0)
		rotate_dx += 360.0;
	m_rotateLineEdit->setText(QString::number(-1.0 * rotate_dx, 'f', 1));

	QPointF pos(0.0, 0.0);
	if (m_triangleButton->isChecked())
		// get the triangle's center position from the scene
		pos = selectedTriangle->mapFromScene(
					selectedTriangle->sceneBoundingRect().center());
	else if (m_markButton->isChecked())
		// center on the marker
		pos = selectedTriangle->mapFromScene(
				selectedTriangle->editor()->mark()->center());

	selectedTriangle->rotate(deg, pos);
}

void EditTriangleWidget::triangleHFlippedAction()
{
	if (m_triangleButton->isChecked())
		selectedTriangle->flipHorizontally();
	else
	{
		QPointF pos(0.0, 0.0);
		if (m_markButton->isChecked())
			pos = selectedTriangle->mapFromScene(
				selectedTriangle->editor()->mark()->center());
		selectedTriangle->flipHorizontally(pos);
	}
}

void EditTriangleWidget::triangleVFlippedAction()
{

	if (m_triangleButton->isChecked())
		selectedTriangle->flipVertically();
	else
	{
		QPointF pos(0.0, 0.0);
		if (m_markButton->isChecked())
			pos = selectedTriangle->mapFromScene(
				selectedTriangle->editor()->mark()->center());
		selectedTriangle->flipVertically(pos);
	}
}

void EditTriangleWidget::sceneScaledAction()
{
	int value = m_sceneScaleSlider->dx();
	double dn;

	if (value > 0)
		dn = 1.05;
	else if (value < 0)
		dn = 0.95;
	else
		return;

	selectedTriangle->basis()->scale(dn, dn);
}

void EditTriangleWidget::toggleGridAction(bool checked)
{
	triangleScene->setGridVisible(checked);
}

void EditTriangleWidget::showEvent(QShowEvent* e)
{
	// quack quack, better to just pass in the ref to the figureeditor...
	m_gridCheckBox->blockSignals(true);
	m_gridCheckBox->setChecked(dynamic_cast<FigureEditor*>
			(selectedTriangle->scene())->gridVisible());
	m_gridCheckBox->blockSignals(false);
}


void EditTriangleWidget::gridColorSelectAction()
{
	QColor c = QColorDialog::getColor(triangleScene->gridColor(), this);
	if (c.isValid())
		triangleScene->setGridColor(c);
}

void EditTriangleWidget::bgColorSelectAction()
{
	QColor c = QColorDialog::getColor(triangleScene->bgColor(), this);
	if (c.isValid())
		triangleScene->setbgColor(c);
}

void EditTriangleWidget::triangleSelectedSlot(Triangle* t)
{
	selectedTriangle = t;
	triangleScene = dynamic_cast<FigureEditor*>(selectedTriangle->scene());
	rotate_dx = 0.0;
	scale_dx  = 1.0;
	m_rotateLineEdit->setText("0.0");
	m_scaleLineEdit->setText("1.0");


	m_finalCheckBox->blockSignals(true);
	if (genome_ptr->final_xform_enable
		   && genome_ptr->final_xform_index
		   == selectedTriangle->editor()->selectedTriangleIndex())
		m_finalCheckBox->setCheckState(Qt::Checked);
	else
		m_finalCheckBox->setCheckState(Qt::Unchecked);
	m_finalCheckBox->blockSignals(false);
}


void EditTriangleWidget::moveTriangleLeftAction()
{
	selectedTriangle->moveTransformBy(m_multiplierEditor->value()*(-1.0),0.0);
}


void EditTriangleWidget::moveTriangleRightAction()
{
	selectedTriangle->moveTransformBy(m_multiplierEditor->value(),0.0);
}



void EditTriangleWidget::moveTriangleUpAction()
{
	selectedTriangle->moveTransformBy(0.0, m_multiplierEditor->value());
}



void EditTriangleWidget::moveTriangleDownAction()
{
	selectedTriangle->moveTransformBy(0.0, m_multiplierEditor->value()*(-1.0));
}


void EditTriangleWidget::rotateTextEditedSlot()
{
	double deg = m_rotateLineEdit->text().toDouble();
	if (deg == 0.0)
		return;

	QPointF pos(0.0, 0.0);
	if (m_triangleButton->isChecked())
		// get the triangle's center position from the scene
		pos = selectedTriangle->mapFromScene(
					selectedTriangle->sceneBoundingRect().center());
	else if (m_markButton->isChecked())
		pos = selectedTriangle->mapFromScene(
				selectedTriangle->editor()->mark()->center());

	rotate_dx = 0.0;
	selectedTriangle->rotate(deg, pos);
}


void EditTriangleWidget::scaleTextEditedSlot()
{
	double scale = m_scaleLineEdit->text().toDouble();
	if (scale == 1.0)
		return;

	QPointF pos(0.0, 0.0);
	if (m_triangleButton->isChecked())
		// get the triangle's center position from the scene
		pos = selectedTriangle->mapFromScene(
					selectedTriangle->sceneBoundingRect().center());
	else if (m_markButton->isChecked())
		pos = selectedTriangle->mapFromScene(
				selectedTriangle->editor()->mark()->center());

	scale_dx = 0.0;
	selectedTriangle->scale(scale, scale, pos);
}


void EditTriangleWidget::toggleMarkAction(bool flag)
{
	selectedTriangle->editor()->setMarkVisible(flag);
}


void EditTriangleWidget::finalStateChangedSlot(int state)
{
	if (state == Qt::Checked)
	{
		genome_ptr->final_xform_enable = 1;
		genome_ptr->final_xform_index = selectedTriangle->editor()->selectedTriangleIndex();
	}
	else
	{
		genome_ptr->final_xform_enable = 0;
		genome_ptr->final_xform_index = -1;
	}
	emit dataChanged();
}

