/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef TRIANGLE_H
#define TRIANGLE_H

#include "basistriangle.h"

class NodeItem;
class FigureEditor;

typedef QList<NodeItem*>           TriangleNodes;
typedef QListIterator<NodeItem*>   TriangleNodesIterator;

class Triangle : public QGraphicsPolygonItem
{
	private:
		FigureEditor* canvas;
		flam3_xform* m_xform;
		BasisTriangle* basisTriangle;
		TriangleCoords cList;
		TriangleNodes  nList;
		QRectF m_boundingRect;
		static uint zpos;
		double m_scaled;
		double m_rotated;

	public:
		Triangle( FigureEditor*, flam3_xform*, BasisTriangle* );
		~Triangle();
		void addNode( NodeItem* );
		flam3_xform* xform();
		void moveEdges();
		void moveBy(double, double);
		void setPoints(TriangleCoords&);
		void moveToFront();
		void show();
		int type() const;
		int nextZPos();
		QMatrix* getCoordinateTransform();
		TriangleNodes& getNodes();
		TriangleCoords getCoords();
		void coordsToXForm();
		QRectF boundingRect();
		void flipHorizontally();
		void flipVertically();
		void flipHorizontally(QPointF);
		void flipVertically(QPointF);
		void scale(double, double, QPointF);
		void rotate(double, QPointF);
		BasisTriangle* basis() const;
		void setNodeColor(const QColor&, const QColor&);
		FigureEditor* editor() const;
		void moveTransformBy(double dx, double dy);

		static const int RTTI = 349495;

	protected:
		void wheelEvent(QGraphicsSceneWheelEvent*);

	private:
		void adjustSceneRect();
};


#include "nodeitem.h"
#include "xfedit.h"

#endif
