/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "trianglecoordswidget.h"
#include "logger.h"

TriangleCoordsWidget::TriangleCoordsWidget(GenomeVector* gen, QWidget* parent)
	: QWidget(parent), genome(gen)
{
	setupUi(this);

	connect(m_AxLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(fieldEditedAction()));
	connect(m_AyLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(fieldEditedAction()));
	connect(m_BxLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(fieldEditedAction()));
	connect(m_ByLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(fieldEditedAction()));
	connect(m_CxLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(fieldEditedAction()));
	connect(m_CyLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(fieldEditedAction()));

	connect(m_aLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(field2EditedAction()));
	connect(m_bLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(field2EditedAction()));
	connect(m_cLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(field2EditedAction()));
	connect(m_dLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(field2EditedAction()));
	connect(m_eLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(field2EditedAction()));
	connect(m_fLineEdit, SIGNAL(textEdited(const QString&)),
			this, SLOT(field2EditedAction()));
}

void TriangleCoordsWidget::setGenome(int n)
{
	genome_offset = n;
}

#define genome_ptr (genome->selectedGenome())

void TriangleCoordsWidget::updateFormData()
{
	if (isHidden())
		return;

	logFine("TriangleCoordsWidget::updateFormData : updating");
	TriangleCoords c = selectedTriangle->getCoords();

	m_AxLineEdit->setText(QString::number(c[0].x(), 'f', 4));
	m_AyLineEdit->setText(QString::number(c[0].y(), 'f', 4));
	m_BxLineEdit->setText(QString::number(c[1].x(), 'f', 4));
	m_ByLineEdit->setText(QString::number(c[1].y(), 'f', 4));
	m_CxLineEdit->setText(QString::number(c[2].x(), 'f', 4));
	m_CyLineEdit->setText(QString::number(c[2].y(), 'f', 4));

	flam3_xform* xform = selectedTriangle->xform();
	m_aLineEdit->setText(QString::number(xform->c[0][0] , 'f', 4));
	m_bLineEdit->setText(QString::number(xform->c[1][0] , 'f', 4));
	m_cLineEdit->setText(QString::number(xform->c[2][0] , 'f', 4));
	m_dLineEdit->setText(QString::number(xform->c[0][1] , 'f', 4));
	m_eLineEdit->setText(QString::number(xform->c[1][1] , 'f', 4));
	m_fLineEdit->setText(QString::number(xform->c[2][1] , 'f', 4));
}

void TriangleCoordsWidget::fieldEditedAction()
{
	logFine("TriangleCoordsWidget::fieldEditedAction : setting coords");
	TriangleCoords c = selectedTriangle->getCoords();
	c[0].setX(m_AxLineEdit->text().toDouble());
	c[0].setY(m_AyLineEdit->text().toDouble());
	c[1].setX(m_BxLineEdit->text().toDouble());
	c[1].setY(m_ByLineEdit->text().toDouble());
	c[2].setX(m_CxLineEdit->text().toDouble());
	c[2].setY(m_CyLineEdit->text().toDouble());
	selectedTriangle->setPoints(c);
	selectedTriangle->coordsToXForm();

	// notify of scene change.  could also just call triangleModifiedAction
	// and not emit, but this keeps widgets from updating unnecessarily
	selectedTriangle->editor()->blockSignals(true);
	selectedTriangle->editor()->triangleModifiedAction(selectedTriangle);
	selectedTriangle->editor()->blockSignals(false);
	emit dataChanged();
}

void TriangleCoordsWidget::field2EditedAction()
{
	logFine("TriangleCoordsWidget::field2EditedAction : setting values");
	flam3_xform* xform = selectedTriangle->xform();
	xform->c[0][0] = m_aLineEdit->text().toDouble();
	xform->c[1][0] = m_bLineEdit->text().toDouble();
	xform->c[2][0] = m_cLineEdit->text().toDouble();
	xform->c[0][1] = m_dLineEdit->text().toDouble();
	xform->c[1][1] = m_eLineEdit->text().toDouble();
	xform->c[2][1] = m_fLineEdit->text().toDouble();

	TriangleCoords c = selectedTriangle->basis()->getCoords(xform);
	selectedTriangle->setPoints(c);
	selectedTriangle->editor()->blockSignals(true);
	selectedTriangle->editor()->triangleModifiedAction(selectedTriangle);
	selectedTriangle->editor()->blockSignals(false);
	emit dataChanged();
}

void TriangleCoordsWidget::triangleSelectedSlot(Triangle* t)
{
	logFine(QString("TriangleCoordsWidget::triangleSelectedSlot : t=0x%1")
			.arg((uint)t, 0, 16));
	selectedTriangle = t;
	updateFormData();
}


void TriangleCoordsWidget::triangleModifiedSlot(Triangle* t)
{
	logFine(QString("TriangleCoordsWidget::triangleModifiedSlot : t=0x%1")
			.arg((uint)t, 0, 16));
	if (t == selectedTriangle)
		updateFormData();
}
